package me.Destro168.ConfigManagers;

import java.io.File;
import java.io.IOException;
import java.util.ConcurrentModificationException;
import java.util.logging.Level;

import me.Destro168.FC_Suite_Shared.FC_Suite_Shared;

import org.bukkit.Bukkit;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.configuration.file.YamlConfiguration;

public class SharedPlayerProfileManager 
{
	private FC_Suite_Shared plugin;
	private FileConfiguration config;
    private File playerFile;
	private String name;
	
	public SharedPlayerProfileManager(String name_) 
	{
		//Variable Declarations
		plugin = FC_Suite_Shared.plugin;
		name = name_;
		playerFile = null;
		
		//Load up the custom config.
		reloadCustomConfig();
	}
	
	public String getConfigFolderPath()
	{
		return plugin.getDataFolder().getAbsolutePath();
	}
	
	public void reloadArgumentConfig(String pluginDataFolderAbsolutePath)
	{
		if (name.equals(""))
		{
			plugin.getLogger().log(Level.SEVERE, "Player name not properly assigned in code.");
			return;
		}
		
		//Set it to the same config to get the path.
        playerFile = new File(pluginDataFolderAbsolutePath, name + ".yml");
        
        //Load up the new configuration file.
        config = YamlConfiguration.loadConfiguration(playerFile);
	}
	
	public void reloadCustomConfig() 
	{
		if (name.equals(""))
		{
			plugin.getLogger().log(Level.SEVERE, "Player name not properly assigned in code.");
			return;
		}
		
		//Set it to the same config to get the path.
        if (playerFile == null)
        	playerFile = new File(getConfigFolderPath(), name + ".yml");
        
        //Load up the new configuration file.
        config = YamlConfiguration.loadConfiguration(playerFile);
    }
	
    public synchronized void saveCustomConfig() 
    {
		//Return if the configs are null.
        if (config == null || playerFile == null)
        	return;
        
        //Attempt to save the files.
        try {
        	config.save(playerFile);
        } catch (IOException ex) {
            plugin.getLogger().log(Level.SEVERE, "Could not save config to " + playerFile, ex);
        } catch (ConcurrentModificationException e) {
        	Bukkit.getScheduler().scheduleAsyncDelayedTask(plugin, new Runnable() { public void run() { saveCustomConfig(); }},1); }
    }
    
    //Clear specific player data.
    public synchronized void clearRpgData()
    {
    	try { config.set("FC_Rpg", null); saveCustomConfig(); } catch (ConcurrentModificationException e) {
			Bukkit.getScheduler().scheduleSyncDelayedTask(plugin, new Runnable() { public void run() { clearRpgData(); }},1); }
    }
    
    /*
    Get/Sets for:
    Long, Int, String, Double, Boolean
    */
    
    //Long
    public synchronized void set(final String field, final long x)
    {
    	try { config.set(field, x); saveCustomConfig(); } catch (ConcurrentModificationException e) {
			Bukkit.getScheduler().scheduleSyncDelayedTask(plugin, new Runnable() { public void run() { set(field, x); }},1); }
    }

    public Long getLong(final String field)
    {
    	reloadCustomConfig();
    	return config.getLong(field);
    }
    
    //String
    public synchronized void set(final String field, final String x)
    {
    	try { config.set(field, x); saveCustomConfig(); } catch (ConcurrentModificationException e) {
			Bukkit.getScheduler().scheduleSyncDelayedTask(plugin, new Runnable() { public void run() { set(field, x); }},1); }
    }

    public String getString(final String field)
    {
    	reloadCustomConfig();
    	return config.getString(field);
    }
    
    //Double
    public synchronized void set(final String field, final double x)
    {
    	try { config.set(field, x); saveCustomConfig(); } catch (ConcurrentModificationException e) {
			Bukkit.getScheduler().scheduleSyncDelayedTask(plugin, new Runnable() { public void run() { set(field, x); }},1); }
    }
    
    public double getDouble(final String field)
    {
    	reloadCustomConfig();
    	return config.getDouble(field);
    }
    
    //Boolean
    public synchronized void set(final String field, final boolean x)
    {
    	try { config.set(field, x); saveCustomConfig(); } catch (ConcurrentModificationException e) {
			Bukkit.getScheduler().scheduleSyncDelayedTask(plugin, new Runnable() { public void run() { set(field, x); }},1); }
    }
    
    public boolean getBoolean(final String field)
    {
    	reloadCustomConfig();
    	return config.getBoolean(field);
    }
    
    //Int
    public synchronized void set(final String field, final int x)
    {
    	try { config.set(field, x); saveCustomConfig(); } catch (ConcurrentModificationException e) {
			Bukkit.getScheduler().scheduleSyncDelayedTask(plugin, new Runnable() { public void run() { set(field, x); }},1); }
    }
    
    public int getInt(final String field)
    {
    	reloadCustomConfig();
    	return config.getInt(field);
    }
}









